@@ -82,7 +82,7 @@ Dispatcher::CompidentType Dispatcher::mapCompNext(const std::string& vhost,
     urlMapCacheType::const_iterator um = urlMapCache.find(cacheKey);
     if (um != urlMapCache.end())
     {
-      log_debug("vhost <" << vhost << "> url <" << compUrl << "> match regex <" << pos->first.getRegex() << "> (cached) => " << um->second.ci);
+      log_debug("vhost <" << vhost << "> url <" << compUrl << "> match regex <" << um->second.pos->first.getRegex() << "> (cached) => " << um->second.ci);
       pos = um->second.pos;
       return um->second.ci;
     }
@@ -108,7 +108,7 @@ Dispatcher::CompidentType Dispatcher::mapCompNext(const std::string& vhost,
 
       if (maxUrlMapCache > 0)
       {
-        // clear cache after maxUrlMapCache distict requests
+        // clear cache after maxUrlMapCache distinct requests
         if (urlMapCache.size() >= maxUrlMapCache)
         {
           log_warn("clear url-map-cache");
