@@ -1,20 +1,29 @@
-/* job.cpp
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
- * This program is free software; you can redistribute it and/or
+ * 
+ * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
+ * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -46,7 +55,6 @@ namespace tnt
   {
     parser.reset();
     request.clear();
-    touch();
   }
 
   int Job::msecToTimeout(time_t currentTime) const
@@ -64,25 +72,26 @@ namespace tnt
   ////////////////////////////////////////////////////////////////////////
   // Tcpjob
   //
+
+  std::string Tcpjob::getPeerIp() const
+  {
+    return socket.getPeerAddr();
+  }
+
+  std::string Tcpjob::getServerIp() const
+  {
+    return socket.getSockAddr();
+  }
+
+  bool Tcpjob::isSsl() const
+  {
+    return false;
+  }
+
   void Tcpjob::accept()
   {
-    log_debug("accept");
     socket.accept(listener);
-
-    fcntl(socket.getFd(), F_SETFD, FD_CLOEXEC);
-
-    struct sockaddr_storage s = socket.getSockAddr();
-    struct sockaddr_storage sockaddr;
-    memcpy(&sockaddr, &s, sizeof(sockaddr));
-
-    char buffer[INET6_ADDRSTRLEN];
-    log_debug("connection accepted from "
-      << inet_ntop(AF_INET6, &(socket.getPeeraddr()), buffer, sizeof(buffer)));
-
-    getRequest().setPeerAddr(socket.getPeeraddr());
-    getRequest().setServerAddr(sockaddr);
-    getRequest().setSsl(false);
-    touch();
+    log_debug("connection accepted from " << getPeerIp());
   }
 
   void Tcpjob::regenerateJob()
@@ -95,12 +104,12 @@ namespace tnt
 
   std::iostream& Tcpjob::getStream()
   {
-    if (socket.cxxtools::net::Socket::bad())
+    if (!socket.isConnected())
     {
       try
       {
         accept();
-        log_debug("connection accepted");
+        touch();
       }
       catch (const std::exception& e)
       {
@@ -134,30 +143,34 @@ namespace tnt
   ////////////////////////////////////////////////////////////////////////
   // SslTcpjob
   //
+
+  std::string SslTcpjob::getPeerIp() const
+  {
+    return socket.getPeerAddr();
+  }
+
+  std::string SslTcpjob::getServerIp() const
+  {
+    return socket.getSockAddr();
+  }
+
+  bool SslTcpjob::isSsl() const
+  {
+    return true;
+  }
+
   void SslTcpjob::accept()
   {
-    log_debug("accept (ssl)");
-    socket.accept(listener);
-    log_debug("connection accepted (ssl)");
+    socket.accept(listener, cxxtools::net::TcpSocket::DEFER_ACCEPT);
   }
 
   void SslTcpjob::handshake()
   {
     socket.handshake(listener);
-    log_debug("ssl handshake ready");
 
     fcntl(socket.getFd(), F_SETFD, FD_CLOEXEC);
 
-    struct sockaddr_storage s = socket.getSockAddr();
-    struct sockaddr_storage sockaddr;
-    memcpy(&sockaddr, &s, sizeof(sockaddr));
-
-    getRequest().setPeerAddr(socket.getPeeraddr());
-    getRequest().setServerAddr(sockaddr);
-    getRequest().setSsl(true);
-
     setRead();
-    touch();
   }
 
   void SslTcpjob::regenerateJob()
@@ -170,16 +183,16 @@ namespace tnt
 
   std::iostream& SslTcpjob::getStream()
   {
-    if (socket.cxxtools::net::Socket::bad())
+    if (!socket.isConnected())
     {
       try
       {
         accept();
-        log_debug("connection accepted");
+        touch();
       }
       catch (const std::exception& e)
       {
-        log_debug("error occured in accept: " << e.what());
+        log_debug("exception occured in accept (ssl): " << e.what());
         regenerateJob();
         throw;
       }
@@ -215,7 +228,6 @@ namespace tnt
   //
   void Jobqueue::put(JobPtr j, bool force)
   {
-    log_debug("Jobqueue::put");
     j->touch();
 
     cxxtools::MutexLock lock(mutex);
@@ -229,14 +241,10 @@ namespace tnt
       }
     }
 
-    log_debug("jobs.push");
     jobs.push_back(j);
 
     if (waitThreads == 0)
-    {
-      log_debug("no waiting threads left");
       noWaitThreads.signal();
-    }
 
     notEmpty.signal();
   }
@@ -248,25 +256,19 @@ namespace tnt
     // wait, until a job is available
     ++waitThreads;
 
-    log_debug("wait for job (" << jobs.size() << " jobs available)");
-
     while (jobs.empty())
       notEmpty.wait(lock);
 
     --waitThreads;
 
-    log_debug("Jobqueue: fetch job " << waitThreads << " waiting threads left; " << jobs.size() << " jobs in queue");
-
     // take next job (queue is locked)
     JobPtr j = jobs.front();
     jobs.pop_front();
 
     // if there are threads waiting, wake another
     if (!jobs.empty() && waitThreads > 0)
-    {
-      log_debug("signal another thread");
       notEmpty.signal();
-    }
+
     notFull.signal();
 
     return j;
