@@ -1,24 +1,35 @@
-/* messageheaderparser.cpp
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
 #include <tnt/messageheaderparser.h>
+#include <tnt/httperror.h>
+#include <tnt/http.h>
 #include <cctype>
 #include <cxxtools/log.h>
 
@@ -44,9 +55,9 @@ namespace tnt
   {
     if (ch >= 33 && ch <= 126 && ch != ':')
     {
-      fieldname.clear();
-      fieldname.reserve(16);
-      fieldname += ch;
+      fieldnamePtr = headerdataPtr;
+      checkHeaderspace(1);
+      *headerdataPtr++ = ch;
       SET_STATE(state_fieldname);
     }
     else if (ch == '\n')
@@ -77,13 +88,25 @@ namespace tnt
   {
     if (ch == ':')            // Field-name:
     {
-      fieldname += ch;
+      checkHeaderspace(2);
+      *headerdataPtr++ = ch;
+      *headerdataPtr++ = '\0';
+      fieldbodyPtr = headerdataPtr;
       SET_STATE(state_fieldbody0);
     }
     else if (ch >= 33 && ch <= 126)
-      fieldname += ch;
+    {
+      checkHeaderspace(1);
+      *headerdataPtr++ = ch;
+    }
     else if (std::isspace(ch))
+    {
+      checkHeaderspace(2);
+      *headerdataPtr++ = ':';
+      *headerdataPtr++ = '\0';
+      fieldbodyPtr = headerdataPtr;
       SET_STATE(state_fieldnamespace);
+    }
     else
     {
       log_warn("invalid character " << chartoprint(ch) << " in fieldname");
@@ -97,12 +120,6 @@ namespace tnt
   {
     if (ch == ':')                   // "Field-name :"
       SET_STATE(state_fieldbody0);
-    else if (ch >= 33 && ch <= 126)  // "Field-name blah..."
-    {
-      fieldbody.reserve(32);
-      fieldbody += ch;
-      SET_STATE(state_fieldbody);
-    }
     else if (!std::isspace(ch))
     {
       log_warn("invalid character " << chartoprint(ch) << " in fieldname-space");
@@ -115,14 +132,21 @@ namespace tnt
   bool Messageheader::Parser::state_fieldbody0(char ch)
   {
     if (ch == '\r')
+    {
+      checkHeaderspace(1);
+      *headerdataPtr++ = '\0';
       SET_STATE(state_fieldbody_cr);
+    }
     else if (ch == '\n')
+    {
+      checkHeaderspace(1);
+      *headerdataPtr++ = '\0';
       SET_STATE(state_fieldbody_crlf);
+    }
     else if (!std::isspace(ch))
     {
-      fieldbody.clear();
-      fieldbody.reserve(32);
-      fieldbody += ch;
+      checkHeaderspace(1);
+      *headerdataPtr++ = ch;
       SET_STATE(state_fieldbody);
     }
     return false;
@@ -131,11 +155,22 @@ namespace tnt
   bool Messageheader::Parser::state_fieldbody(char ch)
   {
     if (ch == '\r')
+    {
+      checkHeaderspace(1);
+      *headerdataPtr++ = '\0';
       SET_STATE(state_fieldbody_cr);
+    }
     else if (ch == '\n')
+    {
+      checkHeaderspace(1);
+      *headerdataPtr++ = '\0';
       SET_STATE(state_fieldbody_crlf);
+    }
     else
-      fieldbody += ch;
+    {
+      checkHeaderspace(1);
+      *headerdataPtr++ = ch;
+    }
     return false;
   }
 
@@ -158,8 +193,8 @@ namespace tnt
       SET_STATE(state_end_cr);
     else if (ch == '\n')
     {
-      log_debug("header " << fieldname << ": " << fieldbody);
-      switch (header.onField(fieldname, fieldbody))
+      log_debug("header " << fieldnamePtr << ": " << fieldbodyPtr);
+      switch (header.onField(fieldnamePtr, fieldbodyPtr))
       {
         case OK:
         case END:  return true;
@@ -168,18 +203,20 @@ namespace tnt
                    log_warn("invalid character " << chartoprint(ch) << " in fieldbody");
                    break;
       }
-      fieldname.clear();
-      fieldbody.clear();
+
       return true;
     }
     else if (std::isspace(ch))
     {
-      fieldbody += ch;
+      // continuation line
+      checkHeaderspace(1);
+      *(headerdataPtr - 1) = '\n';
+      *headerdataPtr++ = ch;
       SET_STATE(state_fieldbody);
     }
     else if (ch >= 33 && ch <= 126)
     {
-      switch (header.onField(fieldname, fieldbody))
+      switch (header.onField(fieldnamePtr, fieldbodyPtr))
       {
         case OK:   SET_STATE(state_fieldname);
                    break;
@@ -189,10 +226,10 @@ namespace tnt
         case END:  return true;
                    break;
       }
-      fieldbody.clear();
-      fieldname.clear();
-      fieldname.reserve(16);
-      fieldname += ch;
+
+      fieldnamePtr = headerdataPtr;
+      checkHeaderspace(1);
+      *headerdataPtr++ = ch;
     }
     return false;
   }
@@ -201,14 +238,13 @@ namespace tnt
   {
     if (ch == '\n')
     {
-      if (header.onField(fieldname, fieldbody) == FAIL)
+      if (header.onField(fieldnamePtr, fieldbodyPtr) == FAIL)
       {
-        log_warn("invalid header " << fieldname << ' ' << fieldbody);
+        log_warn("invalid header " << fieldnamePtr << ' ' << fieldbodyPtr);
         failedFlag = true;
       }
 
-      fieldname.clear();
-      fieldbody.clear();
+      *headerdataPtr = '\0';
       return true;
     }
     else
@@ -220,9 +256,16 @@ namespace tnt
     return false;
   }
 
+  void Messageheader::Parser::checkHeaderspace(unsigned chars) const
+  {
+    if (headerdataPtr + chars >= header.rawdata + sizeof(header.rawdata))
+      throw HttpError(HTTP_REQUEST_ENTITY_TOO_LARGE, "header too large");
+  }
+
   void Messageheader::Parser::reset()
   {
     failedFlag = false;
+    headerdataPtr = header.rawdata;
     SET_STATE(state_0);
   }
 }
