@@ -1,20 +1,29 @@
-/* scopemanager.cpp
+/*
  * Copyright (C) 2003-2006 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -24,18 +33,15 @@
 #include "tnt/httpreply.h"
 #include <cxxtools/log.h>
 #include <cxxtools/md5stream.h>
+#include <pthread.h>
 #include <stdlib.h>
 
 log_define("tntnet.scopemanager")
 
 namespace tnt
 {
-  static const char sessionCookiePrefix[] = "tntnet.";
-
   Scope* ScopeManager::getApplicationScope(const std::string& appname)
   {
-    log_debug("getApplicationScope(\"" << appname << "\")");
-
     cxxtools::MutexLock lock(applicationScopesMutex);
 
     scopes_type::iterator it = applicationScopes.find(appname);
@@ -73,8 +79,6 @@ namespace tnt
 
   bool ScopeManager::hasSessionScope(const std::string& sessioncookie)
   {
-    log_debug("hasSessionScope(\"" << sessioncookie << "\")");
-
     cxxtools::MutexLock lock(sessionScopesMutex);
     sessionscopes_type::iterator it = sessionScopes.find(sessioncookie);
     return it != sessionScopes.end();
@@ -82,15 +86,12 @@ namespace tnt
 
   void ScopeManager::putSessionScope(const std::string& sessioncookie, Sessionscope* s)
   {
-    log_debug("putSessionScope " << sessioncookie);
-
     s->addRef();
 
     cxxtools::MutexLock lock(sessionScopesMutex);
     sessionscopes_type::iterator it = sessionScopes.find(sessioncookie);
     if (it != sessionScopes.end())
     {
-      log_debug("sessionscope found - replace");
       it->second->release();
       it->second = s;
     }
@@ -101,13 +102,10 @@ namespace tnt
 
   void ScopeManager::removeApplicationScope(const std::string& appname)
   {
-    log_debug("removeApplicationScope(\"" << appname << "\")");
-
     cxxtools::MutexLock lock(applicationScopesMutex);
     scopes_type::iterator it = applicationScopes.find(appname);
     if (it != applicationScopes.end())
     {
-      log_debug("release applicationscope");
       it->second->release();
       applicationScopes.erase(it);
     }
@@ -115,13 +113,10 @@ namespace tnt
 
   void ScopeManager::removeSessionScope(const std::string& sessioncookie)
   {
-    log_debug("removeSessionScope(\"" << sessioncookie << "\")");
-
     cxxtools::MutexLock lock(sessionScopesMutex);
     sessionscopes_type::iterator it = sessionScopes.find(sessioncookie);
     if (it != sessionScopes.end())
     {
-      log_debug("release sessionscope");
       it->second->release();
       sessionScopes.erase(it);
     }
@@ -130,7 +125,7 @@ namespace tnt
   void ScopeManager::preCall(HttpRequest& request, const std::string& app)
   {
     // check session-cookie
-    std::string currentSessionCookieName = sessionCookiePrefix + app;
+    std::string currentSessionCookieName = app.empty() ? std::string("tntnet") : "tntnet." + app;
     Cookie c = request.getCookie(currentSessionCookieName);
     if (c.getValue().empty())
     {
@@ -141,55 +136,98 @@ namespace tnt
         log_debug("available session " << it->first << " value " << it->second);
         */
 
-      log_debug("session-cookie " << currentSessionCookieName << " not found");
-      request.setSessionScope(0);
+      log_debug("session-cookie " << currentSessionCookieName << " not found - keep session");
     }
     else
     {
       log_debug("session-cookie " << currentSessionCookieName << " found: " << c.getValue());
-      Sessionscope* sessionScope = getSessionScope(c.getValue());
-      if (sessionScope != 0)
+
+      cxxtools::MutexLock lock(sessionScopesMutex);
+
+      Sessionscope* sessionScope;
+
+      sessionscopes_type::iterator it = sessionScopes.find(c.getValue());
+      if (it == sessionScopes.end())
+      {
+        log_debug("session not found - create new");
+        sessionScope = new Sessionscope();
+        sessionScope->addRef();
+        sessionScopes.insert(sessionscopes_type::value_type(c.getValue(), sessionScope));
+      }
+      else
       {
         log_debug("session found");
-        request.setSessionScope(sessionScope);
+        sessionScope = it->second;
+        sessionScope->touch();
       }
+
+      request.setSessionScope(sessionScope);
     }
 
     // set application-scope
     request.setApplicationScope(getApplicationScope(app));
   }
 
-  void ScopeManager::postCall(HttpRequest& request, HttpReply& reply, const std::string& app)
+  void ScopeManager::setSessionId(HttpRequest& request, const std::string& sessionId)
   {
-    std::string currentSessionCookieName = sessionCookiePrefix + app;
+    if (sessionId.empty())
+    {
+      request.setSessionScope(0);
+    }
+    else
+    {
+      Sessionscope* sessionScope = getSessionScope(sessionId);
+      if (sessionScope != 0)
+      {
+        log_debug("session found");
+        request.setSessionScope(sessionScope);
+      }
+    }
+  }
+
+  std::string ScopeManager::postCall(HttpRequest& request, HttpReply& reply, const std::string& app)
+  {
+    std::string currentSessionCookieName = app.empty() ? std::string("tntnet") : "tntnet." + app;
+
+    std::string sessionId;
 
     if (request.hasSessionScope())
     {
       // request has session-scope
-      std::string cookie = request.getCookie(currentSessionCookieName);
-      if (cookie.empty() || !hasSessionScope(cookie))
+      sessionId = request.getCookie(currentSessionCookieName);
+      if (sessionId.empty())
       {
-        // client has no or unknown cookie
+        // client has no sessionId
 
         cxxtools::Md5stream c;
         c << request.getSerial() << '-' << ::pthread_self() << '-' << rand();
-        cookie = c.getHexDigest();
-        log_info("create new session " << cookie);
-        reply.setCookie(currentSessionCookieName, cookie);
-        putSessionScope(cookie, &request.getSessionScope());
+        sessionId = c.getHexDigest();
+        log_info("create new session " << sessionId);
+        reply.setCookie(currentSessionCookieName, sessionId);
+        putSessionScope(sessionId, &request.getSessionScope());
+      }
+      else if (!hasSessionScope(sessionId))
+      {
+        // client has a sessionId but no associated session
+        // this may happen, when tntnet is restarted while the browser has a session
+
+        putSessionScope(sessionId, &request.getSessionScope());
       }
     }
     else
     {
-      std::string cookie = request.getCookie(currentSessionCookieName);
-      if (!cookie.empty())
+      sessionId = request.getCookie(currentSessionCookieName);
+      if (!sessionId.empty())
       {
-        // client has cookie
+        // client has sessionId
         log_debug("clear Cookie " << currentSessionCookieName);
         reply.clearCookie(currentSessionCookieName);
-        removeSessionScope(cookie);
+        removeSessionScope(sessionId);
+        sessionId.clear();
       }
     }
+
+    return sessionId;
   }
 
   void ScopeManager::checkSessionTimeout()
@@ -214,7 +252,6 @@ namespace tnt
       else
         ++it;
     }
-    log_debug(count << " sessions timed out " << sessionScopes.size() << " sessions left");
 
   }
 }
