@@ -1,20 +1,29 @@
-/* tnt/httprequest.h
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -22,20 +31,25 @@
 #define TNT_HTTPREQUEST_H
 
 #include <tnt/httpmessage.h>
-#include <vector>
-#include <netinet/in.h>
+#include <tnt/httpheader.h>
+#include <tnt/socketif.h>
 #include <tnt/contenttype.h>
 #include <tnt/multipart.h>
 #include <tnt/cookie.h>
 #include <tnt/encoding.h>
-#include <cxxtools/query_params.h>
+#include <tnt/query_params.h>
 #include <tnt/scope.h>
+#include <tnt/threadcontext.h>
 #include <locale>
-#include <sys/socket.h>
+#include <vector>
+#include <cxxtools/atomicity.h>
+#include <string>
+#include <cstring>
 
 namespace tnt
 {
   class Sessionscope;
+  class Tntnet;
 
   /// HTTP-Request-message
   class HttpRequest : public HttpMessage
@@ -43,12 +57,15 @@ namespace tnt
     public:
       class Parser;
       friend class Parser;
+      friend class SessionUnlocker;
+      friend class ApplicationUnlocker;
 
       typedef std::vector<std::string> args_type;
 
     private:
       std::string body;
-      std::string method;
+      unsigned methodLen;
+      char method[8];
       std::string url;
       std::string queryString;
 
@@ -57,16 +74,14 @@ namespace tnt
 
       std::string pathinfo;
       args_type args;
-      cxxtools::QueryParams qparam;
+      tnt::QueryParams qparam;
 
-      struct sockaddr_storage peerAddr;
-      struct sockaddr_storage serverAddr;
+      const SocketIf* socketIf;
 
-      Contenttype ct;
+      mutable Contenttype ct;
       Multipart mp;
-      bool ssl;
-      unsigned serial;
-      static unsigned serial_;
+      cxxtools::atomic_t serial;
+      static cxxtools::atomic_t serial_;
       mutable bool locale_init;
       mutable std::string lang;
       mutable std::locale locale;
@@ -79,8 +94,8 @@ namespace tnt
 
       Scope* requestScope;
       Scope* applicationScope;
-      Scope* threadScope;
       Sessionscope* sessionScope;
+      ThreadContext* threadContext;
 
       bool applicationScopeLocked;
       bool sessionScopeLocked;
@@ -91,14 +106,17 @@ namespace tnt
       void releaseApplicationScopeLock();
       void releaseSessionScopeLock();
 
-      void releaseLocks() { releaseSessionScopeLock(); }
+      void releaseLocks() { releaseApplicationScopeLock(); }
 
       mutable std::string peerAddrStr;
       mutable std::string serverAddrStr;
 
+      const Contenttype& getContentTypePriv() const;
+      Tntnet& application;
+
     public:
-      HttpRequest();
-      HttpRequest(const std::string& url);
+      explicit HttpRequest(Tntnet& application_, const SocketIf* socketIf = 0);
+      HttpRequest(Tntnet& application_, const std::string& url, const SocketIf* socketIf = 0);
       HttpRequest(const HttpRequest& r);
       ~HttpRequest();
 
@@ -111,9 +129,13 @@ namespace tnt
       /// sets the body of the message.
       void setBody(const std::string& body_)    { body = body_; }
       /// Returns the http-method (normally GET or POST) of a request.
-      const std::string& getMethod() const      { return method; }
+      std::string getMethod() const             { return method; }
+      const char* getMethod_cstr() const        { return method; }
+      bool isMethodGET() const                  { return std::strcmp(method, "GET") == 0; }
+      bool isMethodPOST() const                 { return std::strcmp(method, "POST") == 0; }
+      bool isMethodHEAD() const                 { return std::strcmp(method, "HEAD") == 0; }
       /// sets the http-method of this request.
-      void setMethod(const std::string& m)      { method = m; }
+      void setMethod(const char* method);
 
       /// returns url with get-parameters.
       std::string getQuery() const
@@ -133,42 +155,31 @@ namespace tnt
       const args_type& getArgs() const             { return args; }
       args_type& getArgs()                         { return args; }
 
-      args_type::const_reference getArgDef(args_type::size_type n,
+      std::string getArgDef(args_type::size_type n,
         const std::string& def = std::string()) const
         { return args.size() > n ? args[n] : def; }
-      args_type::const_reference getArg(args_type::size_type n) const
-                                                   { return args[n]; }
+      const std::string& getArg(args_type::size_type n) const
+        { return args[n]; }
+      std::string getArg(const std::string& name, const std::string& def = std::string()) const;
       args_type::size_type getArgsCount() const    { return args.size(); }
 
       void parse(std::istream& in);
       void doPostParse();
 
-      cxxtools::QueryParams& getQueryParams()               { return qparam; }
-      const cxxtools::QueryParams& getQueryParams() const   { return qparam; }
+      tnt::QueryParams& getQueryParams()               { return qparam; }
+      const tnt::QueryParams& getQueryParams() const   { return qparam; }
+      void setQueryParams(const tnt::QueryParams& q)   { qparam = q; }
 
-      void setPeerAddr(const struct sockaddr_storage& p)
-        { memcpy(&peerAddr, &p, sizeof(peerAddr)); peerAddrStr.clear(); }
-      const struct sockaddr_storage& getPeerAddr() const
-        { return peerAddr; }
-      std::string getPeerIp() const;
+      std::string getPeerIp() const    { return socketIf ? socketIf->getPeerIp()   : std::string(); }
+      std::string getServerIp() const  { return socketIf ? socketIf->getServerIp() : std::string(); }
+      bool isSsl() const  { return socketIf && socketIf->isSsl(); }
 
-      void setServerAddr(const struct sockaddr_storage& p)
-        { memcpy(&serverAddr, &p, sizeof(serverAddr)); serverAddrStr.clear(); }
-      const struct sockaddr_storage& getServerAddr() const
-        { return serverAddr; }
-      std::string getServerIp() const;
-      unsigned short int getServerPort() const
-        { return ntohs(reinterpret_cast <const struct sockaddr_in *> (&serverAddr)->sin_port); }
-
-      void setSsl(bool sw = true)
-        { ssl = sw; }
-      bool isSsl() const
-        { return ssl;  }
-      const Contenttype& getContentType() const  { return ct; }
-      bool isMultipart() const               { return ct.isMultipart(); }
+      const Contenttype& getContentType() const
+        { return ct.getType().empty() && hasHeader(httpheader::contentType) ? getContentTypePriv() : ct; }
+      bool isMultipart() const               { return getContentType().isMultipart(); }
       const Multipart& getMultipart() const  { return mp; }
 
-      unsigned getSerial() const             { return serial; }
+      cxxtools::atomic_t getSerial() const   { return serial; }
 
       const std::locale& getLocale() const;
       const std::string& getLang() const
@@ -190,9 +201,9 @@ namespace tnt
         { return getCookies().getCookie(name); }
 
       const Encoding& getEncoding() const;
-      std::string getUserAgent() const
+      const char* getUserAgent() const
         { return getHeader(httpheader::userAgent); }
-      std::string getHost() const
+      const char* getHost() const
         { return getHeader(httpheader::host); }
 
       const std::string& getUsername() const;
@@ -200,17 +211,17 @@ namespace tnt
       bool verifyPassword(const std::string& password) const;
 
       bool keepAlive() const;
+      bool acceptGzipEncoding() const           { return getEncoding().accept("gzip"); }
 
       void setApplicationScope(Scope* s);
       void setApplicationScope(Scope& s)  { setApplicationScope(&s); }
 
-      void setThreadScope(Scope* s);
-      void setThreadScope(Scope& s)       { setThreadScope(&s); }
-
       void setSessionScope(Sessionscope* s);
       void setSessionScope(Sessionscope& s)      { setSessionScope(&s); }
       void clearSession();
 
+      void setThreadContext(ThreadContext* ctx)    { threadContext = ctx; }
+
       Scope& getRequestScope();
       Scope& getApplicationScope();
       Scope& getThreadScope();
@@ -225,6 +236,12 @@ namespace tnt
       std::string getVirtualHost() const
         { return getHeader(httpheader::host); }
 
+      Tntnet& getApplication()
+        { return application; }
+
+      /// rewind watchdog timer.
+      void touch()                               { threadContext->touch(); }
+
       /// Sets a limit for a maximum request size.
       static void setMaxRequestSize(size_t s)    { maxRequestSize = s; }
       static size_t getMaxRequestSize()          { return maxRequestSize; }
