@@ -1,35 +1,42 @@
-/* tntconfig.cpp
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
 #include <tnt/tntconfig.h>
+#include <tnt/util.h>
 #include <stdexcept>
 #include <fstream>
 #include <stack>
 #include <cctype>
 #include <cxxtools/multifstream.h>
-#include <cxxtools/log.h>
 
 namespace tnt
 {
-  log_define("tntnet.tntconfig")
-
   //////////////////////////////////////////////////////////////////////
   // ConfigParser
   //
@@ -176,10 +183,10 @@ namespace tnt
       if (!*inp)
       {
         delete inp;
-        throw std::runtime_error("cannot open include file " + params[0]);
+        throwRuntimeError("cannot open include file " + params[0]);
       }
       else if (istreams.size() > 5)
-        throw std::runtime_error("too many include-levels");
+        throwRuntimeError("too many include-levels");
 
       istreams.push(inp);
       return true;
@@ -225,7 +232,7 @@ namespace tnt
     }
 
     if (state != state_start)
-      throw std::runtime_error("parse error while reading config");
+      throwRuntimeError("parse error while reading config");
   }
 
   //////////////////////////////////////////////////////////////////////
@@ -233,14 +240,13 @@ namespace tnt
   //
   void Tntconfig::load(const char* configfile)
   {
-    log_debug("load configuration \"" << configfile << '"');
     std::ifstream in(configfile);
     if (!in)
     {
       std::string msg;
       msg = "error opening ";
       msg += configfile;
-      throw std::runtime_error(msg);
+      throwRuntimeError(msg);
     }
     load(in);
   }
@@ -283,30 +289,25 @@ namespace tnt
        const std::string& key,
        const params_type::value_type& def) const
   {
-    log_debug("getValue(\"" << key << "\", \"" << def << "\")");
     for (config_entries_type::const_iterator it = config_entries.begin();
          it != config_entries.end(); ++it)
+    {
       if (it->key == key && it->params.size() > 0)
-      {
-        log_debug("getValue returns \"" << it->params[0] << '"');
         return it->params[0];
-      }
+    }
 
-    log_debug("getValue returns default \"" << def << '"');
     return def;
   }
 
   bool Tntconfig::hasValue(const std::string& key) const
   {
-    log_debug("hasValue(\"" << key << "\")");
     for (config_entries_type::const_iterator it = config_entries.begin();
          it != config_entries.end(); ++it)
+    {
       if (it->key == key && it->params.size() > 0)
-      {
-        log_debug("hasValue returns true");
         return true;
-      }
-    log_debug("hasValue returns false");
+    }
+
     return false;
   }
 
