@@ -1,20 +1,29 @@
-/* tntnet.cpp
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
- * This program is free software; you can redistribute it and/or
+ * 
+ * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
+ * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -24,23 +33,21 @@
 #include "tnt/http.h"
 #include "tnt/httpreply.h"
 #include "tnt/sessionscope.h"
+#include "tnt/tntconfig.h"
+#include "tnt/configurator.h"
+#include "tnt/util.h"
+#ifdef WITH_STRESSJOB
+#include "tnt/stressjob.h"
+#endif
 
-#include <cxxtools/tcpstream.h>
-#include <cxxtools/fork.h>
+#include <cxxtools/net/tcpstream.h>
 #include <cxxtools/log.h>
 
 #include <unistd.h>
+#include <signal.h>
 
 #include <config.h>
 
-#ifndef TNTNET_CONF
-# define TNTNET_CONF "/etc/tntnet.conf"
-#endif
-
-#ifndef TNTNET_PID
-# define TNTNET_PID "/var/run/tntnet.pid"
-#endif
-
 log_define("tntnet.tntnet")
 
 namespace
@@ -62,7 +69,7 @@ namespace
         {
           std::ostringstream msg;
           msg << "invalid number of parameters (" << args.size() << ") in MapUrl";
-          throw std::runtime_error(msg.str());
+          tnt::throwRuntimeError(msg.str());
         }
 
         std::string url = args[0];
@@ -83,7 +90,7 @@ namespace
         {
           std::ostringstream msg;
           msg << "invalid number of parameters (" << args.size() << ") in VMapUrl";
-          throw std::runtime_error(msg.str());
+          tnt::throwRuntimeError(msg.str());
         }
 
         std::string vhost = args[0];
@@ -101,6 +108,11 @@ namespace
       }
     }
   }
+
+  typedef std::set<tnt::Tntnet*> TntnetInstancesType;
+  cxxtools::Mutex allTntnetInstancesMutex;
+  TntnetInstancesType allRunningTntnetInstances;
+
 }
 
 namespace tnt
@@ -113,26 +125,35 @@ namespace tnt
       maxthreads(100),
       threadstartdelay(10),
       timersleep(10),
-      pollerthread(queue)
+      pollerthread(cxxtools::callable(poller, &Poller::run)),
+      poller(queue)
   { }
 
   bool Tntnet::stop = false;
+  cxxtools::Mutex Tntnet::timeStopMutex;
+  cxxtools::Condition Tntnet::timerStopCondition;
+
   
   Tntnet::listeners_type Tntnet::allListeners;
 
   void Tntnet::init(const Tntconfig& config)
   {
-    minthreads = config.getValue<unsigned>("MinThreads", minthreads);
-    maxthreads = config.getValue<unsigned>("MaxThreads", maxthreads);
-    threadstartdelay = config.getValue<unsigned>("ThreadStartDelay", threadstartdelay);
-    timersleep = config.getValue<unsigned>("TimerSleep", timersleep);
-    Worker::setMaxRequestTime(config.getValue<unsigned>("MaxRequestTime", Worker::getMaxRequestTime()));
-    Worker::setEnableCompression(config.getBoolValue("EnableCompression", Worker::getEnableCompression()));
-    queue.setCapacity(config.getValue<unsigned>("QueueSize", queue.getCapacity()));
-    Sessionscope::setDefaultTimeout(config.getValue<unsigned>("SessionTimeout", Sessionscope::getDefaultTimeout()));
-    Listener::setBacklog(config.getValue<int>("ListenBacklog", Listener::getBacklog()));
-    Listener::setListenRetry(config.getValue<int>("ListenRetry", Listener::getListenRetry()));
-    Dispatcher::setMaxUrlMapCache(config.getValue<unsigned>("MaxUrlMapCache", Dispatcher::getMaxUrlMapCache()));
+    Configurator c(*this);
+    c.setMinThreads(        config.getValue("MinThreads",            c.getMinThreads()));
+    c.setMaxThreads(        config.getValue("MaxThreads",            c.getMaxThreads()));
+    c.setThreadStartDelay(  config.getValue("ThreadStartDelay",      c.getThreadStartDelay()));
+    c.setTimerSleep(        config.getValue("TimerSleep",            c.getTimerSleep()));
+    c.setMaxRequestTime(    config.getValue("MaxRequestTime",        c.getMaxRequestTime()));
+    c.setEnableCompression( config.getBoolValue("EnableCompression", c.getEnableCompression()));
+    c.setQueueSize(         config.getValue("QueueSize",             c.getQueueSize()));
+    c.setSessionTimeout(    config.getValue("SessionTimeout",        c.getSessionTimeout()));
+    c.setListenBacklog(     config.getValue("ListenBacklog",         c.getListenBacklog()));
+    c.setListenRetry(       config.getValue("ListenRetry",           c.getListenRetry()));
+    c.setMaxUrlMapCache(    config.getValue("MaxUrlMapCache",        c.getMaxUrlMapCache()));
+    std::string accessLog_ = config.getValue("AccessLog");
+    if (!accessLog_.empty())
+      c.setAccessLog(accessLog_);
+    c.setMaxBackgroundTasks(config.getValue("MaxBackgroundTasks",    c.getMaxBackgroundTasks()));
 
     Tntconfig::config_entries_type configSetEnv;
     config.getConfigValues("SetEnv", configSetEnv);
@@ -166,22 +187,14 @@ namespace tnt
     Comploader::configure(config);
 
     // configure http
-    HttpRequest::setMaxRequestSize(
-      config.getValue("MaxRequestSize", HttpRequest::getMaxRequestSize()));
-    Job::setSocketReadTimeout(
-      config.getValue("SocketReadTimeout", Job::getSocketReadTimeout()));
-    Job::setSocketWriteTimeout(
-      config.getValue("SocketWriteTimeout", Job::getSocketWriteTimeout()));
-    Job::setKeepAliveMax(
-      config.getValue("KeepAliveMax", Job::getKeepAliveMax()));
-    Job::setSocketBufferSize(
-      config.getValue("BufferSize", Job::getSocketBufferSize()));
-    HttpReply::setMinCompressSize(
-      config.getValue("MinCompressSize", HttpReply::getMinCompressSize()));
-    HttpReply::setKeepAliveTimeout(
-      config.getValue("KeepAliveTimeout", HttpReply::getKeepAliveTimeout()));
-    HttpReply::setDefaultContentType(
-      config.getValue("DefaultContentType", HttpReply::getDefaultContentType()));
+    c.setMaxRequestSize(    config.getValue("MaxRequestSize",      c.getMaxRequestSize()));
+    c.setSocketReadTimeout( config.getValue("SocketReadTimeout",   c.getSocketReadTimeout()));
+    c.setSocketWriteTimeout(config.getValue("SocketWriteTimeout",  c.getSocketWriteTimeout()));
+    c.setKeepAliveMax(      config.getValue("KeepAliveMax",        c.getKeepAliveMax()));
+    c.setSocketBufferSize(  config.getValue("BufferSize",          c.getSocketBufferSize()));
+    c.setMinCompressSize(   config.getValue("MinCompressSize",     c.getMinCompressSize()));
+    c.setKeepAliveTimeout(  config.getValue("KeepAliveTimeout",    c.getKeepAliveTimeout()));
+    c.setDefaultContentType(config.getValue("DefaultContentType",  c.getDefaultContentType()));
 
     // initialize listeners
     Tntconfig::config_entries_type configListen;
@@ -191,7 +204,7 @@ namespace tnt
          it != configListen.end(); ++it)
     {
       if (it->params.empty())
-        throw std::runtime_error("empty Listen-entry");
+        throwRuntimeError("empty Listen-entry");
 
       unsigned short int port = 80;
       if (it->params.size() >= 2)
@@ -202,7 +215,7 @@ namespace tnt
         {
           std::ostringstream msg;
           msg << "invalid port " << it->params[1];
-          throw std::runtime_error(msg.str());
+          throwRuntimeError(msg.str());
         }
       }
 
@@ -211,10 +224,12 @@ namespace tnt
       listen(ip, port);
     }
 
-#ifdef USE_SSL
-    // initialize ssl-listener
     Tntconfig::config_entries_type configSslListen;
     config.getConfigValues("SslListen", configSslListen);
+
+#ifdef USE_SSL
+
+    // initialize ssl-listener
     std::string defaultCertificateFile = config.getValue("SslCertificate");
     std::string defaultCertificateKey = config.getValue("SslKey");
 
@@ -222,7 +237,7 @@ namespace tnt
          it != configSslListen.end(); ++it)
     {
       if (it->params.empty())
-        throw std::runtime_error("empty SslListen-entry");
+        throwRuntimeError("empty SslListen-entry");
 
       unsigned short int port = 443;
       if (it->params.size() >= 2)
@@ -233,7 +248,7 @@ namespace tnt
         {
           std::ostringstream msg;
           msg << "invalid port " << it->params[1];
-          throw std::runtime_error(msg.str());
+          throwRuntimeError(msg.str());
         }
       }
 
@@ -245,13 +260,28 @@ namespace tnt
         it->params.size() >= 3 ? it->params[2] : defaultCertificateKey;
 
       if (certificateFile.empty())
-        throw std::runtime_error("Ssl-certificate not configured");
+        throwRuntimeError("Ssl-certificate not configured");
 
       std::string ip(it->params[0]);
 
       sslListen(certificateFile, keyFile, ip, port);
     }
+
+#else
+
+    if (!configSslListen.empty())
+      throwRuntimeError("SslListen is configured but SSL is not compiled into this tntnet");
+
 #endif // USE_SSL
+
+#ifdef WITH_STRESSJOB
+    std::string stressJob = config.getValue("StressJob");
+    if (!stressJob.empty())
+    {
+      log_debug("create stress job for url \"" << stressJob << '"');
+      queue.put(new StressJob(*this, stressJob));
+    }
+#endif
   }
 
   void Tntnet::listen(const std::string& ip, unsigned short int port)
@@ -304,6 +334,14 @@ namespace tnt
     }
 
     // initialize worker-process
+
+    // set FD_CLOEXEC
+    for (listeners_type::iterator it = listeners.begin(); it != listeners.end(); ++it)
+      (*it)->initialize();
+
+    // SIGPIPE must be ignored
+    ::signal(SIGPIPE, SIG_IGN);
+
     // create worker-threads
     log_info("create " << minthreads << " worker threads");
     for (unsigned i = 0; i < minthreads; ++i)
@@ -315,11 +353,16 @@ namespace tnt
 
     // create poller-thread
     log_debug("start poller thread");
-    pollerthread.create();
+    pollerthread.start();
 
     log_debug("start timer thread");
-    cxxtools::MethodThread<Tntnet, cxxtools::AttachedThread> timerThread(*this, &Tntnet::timerTask);
-    timerThread.create();
+    cxxtools::AttachedThread timerThread(cxxtools::callable(*this, &Tntnet::timerTask));
+    timerThread.start();
+
+    {
+      cxxtools::MutexLock lock(allTntnetInstancesMutex);
+      allRunningTntnetInstances.insert(this);
+    }
 
     // mainloop
     cxxtools::Mutex mutex;
@@ -348,36 +391,38 @@ namespace tnt
 
     log_info("stopping Tntnet");
 
-    // join-loop
-    while (!allListeners.empty())
     {
-      listeners_type::value_type s = *allListeners.begin();
-      log_debug("remove listener from listener-list");
-      allListeners.erase(s);
-
-      log_debug("request listener to stop");
-      s->doStop();
-
-      delete s;
-
-      log_debug("listener stopped");
+      cxxtools::MutexLock lock(allTntnetInstancesMutex);
+      allRunningTntnetInstances.erase(this);
     }
 
-    log_info("listeners stopped");
+    log_info("wake listeners");
+    for (listeners_type::iterator it = listeners.begin(); it != listeners.end(); ++it)
+      (*it)->doStop();
 
-    log_info("stop pollerthread");
-    pollerthread.doStop();
+    log_info("stop poller thread");
+    poller.doStop();
     pollerthread.join();
-    log_info("pollerthread stopped");
 
-    while (true)
+    log_info("stop timer thread");
+    timerThread.join();
+
+    if (Worker::getCountThreads() > 0)
     {
-      log_info("wait for worker threads to stop; " << getQueue().getWaitThreadCount() << " left");
-      if (getQueue().getWaitThreadCount() <= 0)
-        break;
-      usleep(100);
+      log_info("wait for " << Worker::getCountThreads() << " worker threads to stop");
+      while (Worker::getCountThreads() > 0)
+      {
+        log_debug("wait for worker threads to stop; " << Worker::getCountThreads() << " left");
+        usleep(100);
+      }
     }
-    log_info("worker threads stopped");
+
+    log_debug("destroy listeners");
+    for (listeners_type::iterator it = listeners.begin(); it != listeners.end(); ++it)
+      delete *it;
+    listeners.clear();
+
+    log_info("all threads stopped");
   }
 
   void Tntnet::setMinThreads(unsigned n)
@@ -399,19 +444,16 @@ namespace tnt
 
     while (true)
     {
-      unsigned c = timersleep;
-      while (c-- > 0 && !stop)
-        sleep(1);
-
-      if (stop)
-        break;
+      {
+        cxxtools::MutexLock timeStopLock(timeStopMutex);
+        if (stop || timerStopCondition.wait(timeStopLock, timersleep * 1000))
+          break;
+      }
 
       getScopemanager().checkSessionTimeout();
       Worker::timer();
     }
 
-    log_info("stopping Tntnet");
-
     queue.noWaitThreads.signal();
     minthreads = maxthreads = 0;
   }
@@ -419,31 +461,7 @@ namespace tnt
   void Tntnet::shutdown()
   {
     stop = true;
-  }
-
-  bool Tntnet::forkProcess()
-  {
-    cxxtools::Fork process;
-    if (process.child())
-    {
-      // 1. child process
-      while (!allListeners.empty())
-      {
-        listeners_type::value_type s = *allListeners.begin();
-        allListeners.erase(s);
-        delete s;
-
-        log_debug("listener stopped");
-      }
-
-      cxxtools::Fork process2;
-      if (process2.child())
-        return true;
-
-      exit(0);
-    }
-
-    return false;
+    timerStopCondition.broadcast();
   }
 
 }
