@@ -1,20 +1,29 @@
-/* mime.cpp
+/*
  * Copyright (C) 2003,2007 Tommi Maekitalo
- *
- * This program is free software; you can redistribute it and/or
+ * 
+ * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
+ * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 #include "mime.h"
@@ -65,9 +74,9 @@ namespace tnt
     tnt::HttpReply& reply, tnt::QueryParams& qparams)
   {
     if (request.getArgs().size() > 0)
-      reply.setContentType(request.getArg(0));
+      reply.setContentType(request.getArg(0).c_str());
     else if (handler)
-      reply.setContentType(handler->getMimeType(request.getPathInfo()));
+      reply.setContentType(handler->getMimeType(request.getPathInfo()).c_str());
 
     // we do not produce any content, so we pass the request
     // to the next handler:
