@@ -1,33 +1,57 @@
-/* main.cpp
+/*
  * Copyright (C) 2007 Tommi Maekitalo
- *
- * This program is free software; you can redistribute it and/or
+ * 
+ * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
+ * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
-#include <iostream>
+#include "tnt/process.h"
 #include "tnt/tntnet.h"
 #include "tnt/tntconfig.h"
+#include "tnt/cmd.h"
+
 #include <cxxtools/log.h>
 #include <cxxtools/loginit.h>
+#include <cxxtools/arg.h>
+
 #include <stdlib.h>
 #include <sys/stat.h>
-#include "tnt/process.h"
+#include <iostream>
+#include <stdexcept>
+
 #include "config.h"
 
+#ifndef TNTNET_CONF
+# define TNTNET_CONF "/etc/tntnet.conf"
+#endif
+
+#ifndef TNTNET_PID
+# define TNTNET_PID "/var/run/tntnet.pid"
+#endif
+
+
 log_define("tntnet.main")
 
 namespace tnt
@@ -87,6 +111,7 @@ namespace tnt
     setRootdir(config.getValue("Chroot"));
     setUser(config.getValue("User"));
     setGroup(config.getValue("Group"));
+    setErrorLog(config.getValue("ErrorLog"));
   }
 
   void TntnetProcess::initializeLogging()
@@ -142,9 +167,35 @@ int main(int argc, char* argv[])
       return 0;
     }
 
-    tnt::TntnetProcess app(argc, argv);
+    cxxtools::Arg<bool> cmd(argc, argv, 'C');
+    if (cmd)
+    {
+      log_init("tntnet.properties");
 
-    app.run();
+      tnt::Cmd cmdapp(std::cout);
+
+      tnt::QueryParams queryParams;
+
+      while (true)
+      {
+        cxxtools::Arg<std::string> param(argc, argv, 'q');
+        if (!param.isSet())
+          break;
+        queryParams.parse_url(param);
+      }
+
+      for (int a = 1; a < argc; ++a)
+      {
+        log_info("calling component <" << argv[a] << '>');
+        cmdapp.call(tnt::Compident(argv[a]), queryParams);
+      }
+    }
+    else
+    {
+      tnt::TntnetProcess app(argc, argv);
+
+      app.run();
+    }
   }
   catch(const std::exception& e)
   {
