@@ -163,6 +163,7 @@ namespace tnt
         state_commente,
         state_compe0,
         state_compe,
+        state_cond0,
         state_cond,
         state_condexpr,
         state_condexpre,  // 70
@@ -201,8 +202,8 @@ namespace tnt
       paramargs_type paramargs;
       unsigned bracket_count = 0;
       std::string scopetype, scopevar, scopeinit;
-      scope_container_type scope_container;
-      scope_type scope;
+      scope_container_type scope_container = application_container;
+      scope_type scope = global_scope;
       bool inComp = false;
       bool inClose = false;
       bool htmlExpr = false;
@@ -219,6 +220,12 @@ namespace tnt
         switch(state)
         {
           case state_html0:
+            if (ch == ' ' || ch == '\t')
+              std::cerr << curfile << ':' << curline + 1 << ": warning: trailing white space after closing tag" << std::endl;
+            else if (ch == '\r')
+              std::cerr << curfile << ':' << curline + 1 << ": warning: trailing cr after closing tag (dos format?)" << std::endl;
+            // no break - continue with state_html
+
           case state_html:
             if (ch == '<')
             {
@@ -319,7 +326,7 @@ namespace tnt
                 html.clear();
               }
               handler.onLine(curline, curfile); //#
-              state = state_cond;
+              state = state_cond0;
             }
             else if (ch == '/')
             {
@@ -906,7 +913,7 @@ namespace tnt
               arg.clear();
               state = state_args0;
               if (ch == '\n')
-                std::cerr << curfile << ':' << curline << ": warning: old syntax: ';' missing" << std::endl;
+                std::cerr << curfile << ':' << curline + 1 << ": warning: old syntax: ';' missing" << std::endl;
             }
             else if (!std::isspace(ch))
             {
@@ -1302,7 +1309,7 @@ namespace tnt
               state = state_callarge;
             else if (ch == '=')
               state = state_callval0;
-            else if (pass_cgi.empty() && ch == '&' || ch == '/' || ch == '>')
+            else if ((pass_cgi.empty() && ch == '&') || ch == '/' || ch == '>')
             {
               log_debug("onCall(\"" << comp << "comp_args (" << comp_args.size() << "), \"" << pass_cgi << "\", paramargs (" << paramargs.size() << "), defarg (" << defarg.size() << "))");
               handler.onCall(comp, comp_args, arg, paramargs, defarg);
@@ -1326,7 +1333,7 @@ namespace tnt
               arg = ch;
               state = state_callarg;
             }
-            else if (pass_cgi.empty() && ch == '&' || ch == '/' || ch == '>')
+            else if ((pass_cgi.empty() && ch == '&') || ch == '/' || ch == '>')
             {
               log_debug("onCall(\"" << comp << "comp_args (" << comp_args.size() << "), \"" << pass_cgi << "\", paramargs (" << paramargs.size() << "), defarg (" << defarg.size() << "))");
               handler.onCall(comp, comp_args, arg, paramargs, defarg);
@@ -1493,6 +1500,14 @@ namespace tnt
             }
             break;
 
+          case state_cond0:
+            if (ch == '?')
+              htmlExpr = true;
+            else
+              cond += ch;
+            state = state_cond;
+            break;
+
           case state_cond:
             if (ch == '?')
               state = state_condexpr;
@@ -1518,7 +1533,8 @@ namespace tnt
             if (ch == '>')
             {
               log_debug("onCondExpression(\"" << cond << ", " << code << "\")");
-              handler.onCondExpr(cond, expr);
+              handler.onCondExpr(cond, expr, htmlExpr);
+              htmlExpr = false;
               cond.clear();
               expr.clear();
               state = state_html;
