@@ -1,20 +1,29 @@
-/* body.cpp
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
- * This program is free software; you can redistribute it and/or
+ * 
+ * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
+ * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -61,9 +70,9 @@ namespace tnt
            it != paramargs.end(); ++it)
       {
         printLine(out);
-        out << "  " << qparam << ".getScope().put(\""
-            << it->first << "\", tnt::createPointerObject("
-            << it->second << "));\n";
+        out << "    " << qparam << ".getScope().put(\""
+            << it->first << "\", ("
+            << it->second << "), false);\n";
       }
 
       printLine(out);
@@ -127,11 +136,11 @@ namespace tnt
       else
       {
         // case: comp.subcomp@lib
-          callByIdent(out,
-                      "tnt::Subcompident(\"" + id.libname + "\", "
-                                   "\"" + id.compname + "\", "
-                                   "\"" + id.subname + "\")",
-                      qparam);
+        callByIdent(out,
+                    "tnt::Subcompident(\"" + id.libname + "\", "
+                                 "\"" + id.compname + "\", "
+                                 "\"" + id.subname + "\")",
+                    qparam);
       }
     }
 
@@ -142,12 +151,12 @@ namespace tnt
 
       if (haveEndCall)
       {
-        out << "  tnt::Component* _tnt_comp" << getNumber() << " = createComp(" << ident << ");\n"
-               "  if (_tnt_comp" << getNumber() << "->call(request, reply, " << qparam << ") != DECLINED)\n"
-               "  {\n";
+        out << "    tnt::Component* _tnt_comp" << getNumber() << " = createComp(" << ident << ");\n"
+               "    if (_tnt_comp" << getNumber() << "->call(request, reply, " << qparam << ") != DECLINED)\n"
+               "    {\n";
       }
       else
-        out << "  callComp(" << ident << ", request, reply, " << qparam << ");\n";
+        out << "    callComp(" << ident << ", request, reply, " << qparam << ");\n";
     }
 
     void BodypartCall::callByExpr(std::ostream& out, const std::string& qparam) const
@@ -157,53 +166,42 @@ namespace tnt
       else if (haveEndCall)
         throw std::runtime_error("end-tag not allowed in call \"" + comp + '"');
 
-      out << "  callComp(" << comp << ", request, reply, " << qparam << ");\n";
+      out << "    callComp(" << comp << ", request, reply, " << qparam << ");\n";
     }
 
     void BodypartCall::getBody(std::ostream& out) const
     {
-      log_debug("getBody: paramargs " << paramargs.size());
+      log_debug("getBody: paramargs " << paramargs.size() << " number=" << getNumber() << " haveEndCall=" << haveEndCall);
       out << "  // <& " << comp << " ...\n";
 
       std::ostringstream o;
       o << "_tnt_cq" << getNumber();
       std::string queryParamName = o.str();
 
-      if (args.empty())
-      {
-        if (pass_cgi.empty())
-        {
-          printLine(out);
-          out << "  tnt::QueryParams " << queryParamName << "(qparam, false);\n";
-          call(out, queryParamName);
-        }
-        else
-        {
-          call(out, pass_cgi);
-        }
-      }
+      if (!haveEndCall)
+        out << "  { \n";
+
+      printLine(out);
+      if (pass_cgi.empty())
+        out << "    tnt::QueryParams " << queryParamName << "(qparam, false);\n";
       else
+        out << "    tnt::QueryParams " << queryParamName << "(" << pass_cgi << ", true);\n";
+
+      for (comp_args_type::const_iterator i = args.begin();
+           i != args.end(); ++i)
       {
         printLine(out);
-        if (pass_cgi.empty())
-          out << "  tnt::QueryParams " << queryParamName << "(qparam, false);\n";
-        else
-          out << "  tnt::QueryParams " << queryParamName << "(" << pass_cgi << ", true);\n";
-
-        for (comp_args_type::const_iterator i = args.begin();
-             i != args.end(); ++i)
-        {
-          printLine(out);
-          out << "  " << queryParamName << ".add( \"" << i->first
-              << "\", tnt::toString("
-              << i->second
-              << ", reply.out().getloc() ) );\n";
-        }
-
-        call(out, queryParamName);
+        out << "    " << queryParamName << ".add( \"" << i->first
+            << "\", tnt::toString("
+            << i->second
+            << ", reply.out().getloc() ) );\n";
       }
 
-      out << "  // &>\n";
+      call(out, queryParamName);
+
+      out << "    // &>\n";
+      if (!haveEndCall)
+        out << "  }\n";
     }
 
     void BodypartEndCall::getBody(std::ostream& out) const
