@@ -1,20 +1,29 @@
-/* component.cpp
+/*
  * Copyright (C) 2005 Tommi Maekitalo
- *
- * This program is free software; you can redistribute it and/or
+ * 
+ * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
+ * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -27,6 +36,19 @@ namespace tnt
     ////////////////////////////////////////////////////////////////////////
     // Component
     //
+
+    std::string Component::getLogCategory() const
+    {
+      if (!logCategory.empty())
+        return logCategory;
+
+      std::string ret = "component." + componentName;
+      std::string::size_type pos;
+      while ((pos = ret.find('/')) != std::string::npos)
+        ret[pos] = '.';
+      return ret;
+    }
+
     void Component::getBody(std::ostream& body, bool linenumbersEnabled) const
     {
       body << "  // <%args>\n";
