@@ -117,12 +117,12 @@ namespace tnt
     }
 
     void Generator::addImage(const std::string& name, const std::string& content,
-        const std::string& mime, time_t c_time)
+        const std::string& mime, time_t c_time_)
     {
       MultiImageType mi;
       mi.name = name;
       mi.mime = mime;
-      mi.c_time = c_time;
+      mi.c_time = c_time_;
       multiImages.push_back(mi);
 
       data.push_back(content);
@@ -262,12 +262,16 @@ namespace tnt
       currentComp = &maincomp;
     }
 
-    void Generator::onCondExpr(const std::string& cond, const std::string& expr)
+    void Generator::onCondExpr(const std::string& cond, const std::string& expr, bool htmlexpr)
     {
       std::ostringstream m;
       printLine(m);
-      m << "  if (" << cond << ")\n"
-           "    reply.sout() << " << expr << ";\n";
+      m << "  if (" << cond << ")\n";
+      if (htmlexpr)
+        m << "    reply.out() << ";
+      else
+        m << "    reply.sout() << ";
+      m << expr << ";\n";
       currentComp->addHtml(m.str());
     }
 
@@ -277,12 +281,12 @@ namespace tnt
     }
 
     void Generator::onScope(scope_container_type container, scope_type scope,
-      const std::string& type, const std::string& var, const std::string& init)
+      const std::string& type, const std::string& var, const std::string& init_)
     {
-      log_debug("onScope type=\"" << type << "\" var=\"" << var << "\" init=\"" << init << '"');
+      log_debug("onScope type=\"" << type << "\" var=\"" << var << "\" init=\"" << init_ << '"');
       tnt::ecppc::Component* comp = (scope == ecpp::page_scope ? &maincomp : currentComp);
 
-      comp->addScopevar(Scopevar(container, scope, type, var, init, curline, curfile));
+      comp->addScopevar(Scopevar(container, scope, type, var, init_, curline, curfile));
     }
 
     void Generator::onInclude(const std::string& file)
@@ -300,10 +304,6 @@ namespace tnt
       externData = true;
     }
 
-    void Generator::endI18n()
-    {
-    }
-
     void Generator::getIntro(std::ostream& out, const std::string& filename) const
     {
       out << "////////////////////////////////////////////////////////////////////////\n"
@@ -328,17 +328,6 @@ namespace tnt
           << "// </%pre>\n";
     }
 
-    void Generator::getNamespaceStart(std::ostream& out) const
-    {
-      out << "namespace\n"
-             "{\n";
-    }
-
-    void Generator::getNamespaceEnd(std::ostream& out) const
-    {
-      out << "} // namespace\n";
-    }
-
     void Generator::getClassDeclaration(std::ostream& out) const
     {
       if (multiImages.empty() && !isRawMode())
@@ -710,18 +699,20 @@ namespace tnt
       getHeaderIncludes(code);
       getCppIncludes(code);
 
-      code << "log_define(\"component." << maincomp.getLogCategory() << "\")\n\n";
+      if (multiImages.empty() && !isRawMode())
+        code << "log_define(\"" << maincomp.getLogCategory() << "\")\n\n";
 
       getPre(code);
-      code << '\n';
 
-      getNamespaceStart(code);
+      code << "\n" 
+              "namespace\n"
+              "{\n";
 
       getClassDeclaration(code);
 
       getCppBody(code);
 
-      getNamespaceEnd(code);
+      code << "} // namespace\n";
     }
   }
 }
