@@ -105,8 +105,6 @@ namespace tnt
 
         void getHeaderIncludes(std::ostream& out) const;
         void getPre(std::ostream& out) const;
-        void getNamespaceStart(std::ostream& out) const;
-        void getNamespaceEnd(std::ostream& out) const;
         void getClassDeclaration(std::ostream& out) const;
 
         void getCppIncludes(std::ostream& out) const;
@@ -133,6 +131,8 @@ namespace tnt
         void enableLinenumbers(bool sw = true)       { linenumbersEnabled = sw; }
         bool isLinenumbersEnabled() const            { return linenumbersEnabled; }
 
+        void setLogCategory(const std::string& cat)  { maincomp.setLogCategory(cat); }
+
         void addImage(const std::string& name, const std::string& content,
             const std::string& mime, time_t c_time);
 
@@ -157,14 +157,13 @@ namespace tnt
         virtual void onComp(const std::string& code);
         virtual void startClose();
         virtual void endClose();
-        virtual void onCondExpr(const std::string& cond, const std::string& expr);
+        virtual void onCondExpr(const std::string& cond, const std::string& expr, bool htmlexpr);
         virtual void onConfig(const std::string& name, const std::string& value);
         virtual void onScope(scope_container_type container, scope_type scope,
           const std::string& type, const std::string& var, const std::string& init);
         virtual void onInclude(const std::string& file);
         virtual void onIncludeEnd(const std::string& file);
         virtual void startI18n();
-        virtual void endI18n();
 
         void getCpp(std::ostream& out, const std::string& filename) const;
     };
